﻿using System;

using BEPUutilities;
using BEPUphysics.Entities;
using BEPUphysics.CollisionShapes.ConvexShapes;

using BEPUphysics.Constraints.TwoEntity.Joints;

namespace BEPUphysicsDemos.Demos
{
    public class TwoDdemo : StandardDemo
    {
        Entity ball;

        public TwoDdemo(DemosGame game)
                : base(game)
        {
            Random ABitOfChaos = new Random();

            game.Camera.Position = new Vector3(0, 20, 150);

            CreateBaseLevel();

            float bpos = 40.0f;
            for (byte buildings = 0; buildings < (ABitOfChaos.Next(10) + 3); buildings++ )
            {
                float width = (float)ABitOfChaos.Next(12) + 4.0f;
                float height = (float)ABitOfChaos.Next(12) + 3.0f;

                bpos += width + 10.0f;

                CreateStructures((byte)(ABitOfChaos.Next(15) + 1), bpos, width, height);
            }

            //CreateStructures(10, 50);
            //CreateStructures(30, 37, 9, 2);
            //CreateStructures(4, 25, 5, 3);
        }


        private void CreateBaseLevel()
        {
            Entity ground = new Entity(new BoxShape(500, 3, 100)) { Position = new Vector3(0,-1.5f,0) };
            Space.Add(ground);

            ball = new Entity(new SphereShape(1.0f), 40.0f) { Position = new Vector3(-30, 4, 0) };
            Matrix3x3 liti = ball.LocalInertiaTensorInverse;
            liti.M11 = liti.M12 = liti.M13 = 0;
            liti.M21 = liti.M22 = liti.M23 = 0;
            ball.LocalInertiaTensorInverse = liti;

            // prevent the ball going too fast
            ball.LinearDamping = 0.33f;
            
            Space.Add(ball);

            PointOnPlaneJoint ballKeeper = new PointOnPlaneJoint(ball, null, new Vector3(1, 1, 0), Vector3.UnitZ, Vector3.Zero);
            Space.Add(ballKeeper);
        }

        /// <summary>
        /// create a tall structures on position X = Position, with no joints between elements
        /// </summary>
        /// <param name="stories">the number of stories to build</param>
        /// <param name="Position">The position on X axis</param>
        /// <param name="width">the distance between two stand, averaged on X position</param>
        /// <param name="height">the height of stands</param>
        private void CreateStructures(byte stories, float Position, float width = 8.0f, float height = 10.0f)
        {
            Random ABitOfChaos = new Random();

            for (byte storyBuilt = 1; storyBuilt <= stories; storyBuilt++)
            {
                float baseheight = (storyBuilt - 1) * (height + 0.2f);

                Entity stand1 = new Entity(new BoxShape(0.5f, height, 0.5f), 100 / stories) { Position = new Vector3(Position - width / 2.0f, baseheight + height / 2.0f, 0) };
                Entity stand2 = new Entity(new BoxShape(0.5f, height, 0.5f), 100 / stories) { Position = new Vector3(Position + width / 2.0f, baseheight + height / 2.0f, 0) };
                Entity floorpanel1 = new Entity(new BoxShape(width + 1, 0.2f, 0.5f), 80 / stories) { Position = new Vector3(Position, baseheight + height + 0.1f, 0) };

                Matrix3x3 liti = stand1.LocalInertiaTensorInverse;
                liti.M11 = liti.M12 = liti.M13 = 0;
                liti.M21 = liti.M22 = liti.M23 = 0;
                stand1.LocalInertiaTensorInverse = liti;

                liti = stand2.LocalInertiaTensorInverse;
                liti.M11 = liti.M12 = liti.M13 = 0;
                liti.M21 = liti.M22 = liti.M23 = 0;
                stand2.LocalInertiaTensorInverse = liti;

                liti = floorpanel1.LocalInertiaTensorInverse;
                liti.M11 = liti.M12 = liti.M13 = 0;
                liti.M21 = liti.M22 = liti.M23 = 0;
                floorpanel1.LocalInertiaTensorInverse = liti;

                Space.Add(stand1);
                Space.Add(stand2);
                Space.Add(floorpanel1);
            }

            Entity piggy = new Entity(new SphereShape(1.0f), 10.0f) { Position = new Vector3(Position + (float)ABitOfChaos.Next((int)(width / 2)), stories * (height + 0.2f), 0) };
            Space.Add(piggy);

        }

        public override void Update(float dt)
        {
            base.Update(dt);
        }


        public override void DrawUI()
        {
            Game.DataTextDrawer.Draw(string.Format("Throw the ball with the help of the grabber\n(does it look familiar ?)\nBall Z={0:F2}", ball.Position.Z), new Microsoft.Xna.Framework.Vector2(20, 20));
            base.DrawUI();
        }


        public override string Name
        {
            get { return "TwoDdemo"; }
        }
    }
}
